from colorama import Fore, Back

def highlight (out, s):
    # get the width of a terminal
    in_code_block = False

    for l in s.split("\n"):
         bs = bytes("{0}\n".format(l), "utf-8")
         # code block
         if in_code_block or l.lstrip().startswith("```"):
             out.write(bytes(Fore.CYAN, "utf-8"))
             out.write(bs)
             out.write(bytes(Fore.RESET, "utf-8"))
             if l.lstrip().startswith("```"):
                 in_code_block = False if in_code_block else True
         # headers
         elif l.lstrip().startswith("#"):
             out.write(bytes(Fore.CYAN, "utf-8"))
             out.write(bs)
             out.write(bytes(Fore.RESET, "utf-8"))
         # quotation
         elif l.lstrip().startswith(">"):
             out.write(bytes(Fore.BLUE, "utf-8"))
             out.write(bs)
             out.write(bytes(Fore.RESET, "utf-8"))
         # un-ordered list
         elif l.lstrip().startswith("* ") or l.lstrip().startswith("-"):
             out.write(bytes(Fore.GREEN, "utf-8"))
             out.write(bs)
             out.write(bytes(Fore.RESET, "utf-8"))
         else:
             out.write(bs)



import csv
from prettytable import PrettyTable

def format (s):
    table = PrettyTable(header=False)
    for l in csv.reader(s.strip().split("\n")):
        try:
           table.add_row(l)
        except Exception: # bad CSV
            return "csv", s
    return "", str(table)



import os
import sys
import chardet
from subprocess import Popen, PIPE, check_output, STDOUT
from azcat.pretty_print import pretty_print

if sys.version_info[0] == 2:
    from __builtin__ import raw_input as input

def load_file (filepath):
    try:
        # detect & convert character encoding
        with open(filepath, "rb") as f:
            data = f.read()

        en = chardet.detect(data[0:1024])["encoding"]
        if en is None:
            s = data.decode("utf-8")
        else:
            s = data.decode(chardet.detect(data[0:1024])["encoding"])
    except IOError as e:
        sys.exit("azcat: cannot open '%s': %s" % (filepath, str(e)))
    except UnicodeDecodeError:
        sys.exit("azcat: file seems a binary file.")

    # confirm if file size is larger than 1MB
    if os.path.getsize(filepath) > 1024*1024:
        if input("file size is big; do you continue? [Y/n]: ") == "n":
            sys.exit()
    return s


def main (args):
    # GNU global
    if args["t"] is not None:
        args["with_pager"] = True
        try:
            output = check_output(["global", "-x", args["t"]]).decode("utf-8")
            if output == "":
                sys.exit("azcat: symbol ``{0}'' not found".format(args["t"]))
            line, file = list(filter(lambda x: x != "", output.split(" ")))[1:3]
            line = int(line)
        except Exception as e:
            sys.exit("azcat: error occurred in global(1)")
    # normal
    else:
        file = args["file"]
        line = 1

    s = load_file(file)
    ext = args.get("f")
    if ext == None:
        ext = os.path.splitext(file)[1][1:]

    if args["with_pager"]:
        p = Popen(["less", "-R", "+{0}g".format(line)], stdin=PIPE)
        try:
            pretty_print(file, s, p.stdin, args["with_formatter"], ext=ext)
            p.stdin = sys.stdin
            p.wait()
        except IOError: # this will raised after the pager existed
            pass
    else:
        pretty_print(file, s, sys.stdout.buffer,
                     args["with_formatter"], ext=ext)



from setuptools import setup, find_packages
import os
import sys

requires = open("requirements.txt").read().split("\n")
readme = open("README.rst").read()

setup(
    name="azcat",
    version="1.0.5",
    description="A alternative to cat(1); specialized for printing files",
    long_description=readme,
    author="Seiya Nuta",
    author_email="nuta@seiya.me",
    url="http://github.com/nuta/azcat",
    packages=find_packages(),
    scripts=["az"],
    install_requires=requires,
    classifiers = [
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "License :: Public Domain",
        "Operating System :: POSIX",
        "Topic :: Utilities"
    ]
)



import os
import mimetypes

def guess_ext_by_filename (filename):
    FILENAMES = {
                  "Makefile":  "mak",
                }

    ext = os.path.splitext(filename)[1].lstrip(".")
    if ext == "":
        for k,v in FILENAMES.items():
            if filename.startswith(k):
                return v
        return ""
    else:
        return ext


def guess_ext_by_contents (s):
    INTERPRETERS = {
                     "python": "py",
                     "ruby":   "rb",
                     "perl":   "pl",
                     "php":    "php",
                     "bash":   "sh",
                     "zsh":    "sh",
                     "sh":     "sh",
                   }

    # guess by shebang
    if isinstance(s, str) and s.startswith("#!"):
        shebang = s.split("\n", 1)[0]
        try:
            interpreter = shebang.split("env ")[1] #!/usr/bin/env python
        except IndexError:
            interpreter = os.path.basename(shebang[2:]) #!/usr/bin/python

        for k,v in INTERPRETERS.items():
            if interpreter.startswith(k):
                return v

    return ""



import os
import json

import pygments
import pygments.lexers
import pygments.formatters

from azcat.guess_ext import guess_ext_by_contents, guess_ext_by_filename

def load_module (type_, name):
    try:
        m = getattr(getattr(__import__("azcat.{0}.{1}".format(type_, name)), type_), name)
    except ImportError:
        return None
    return m

def _load_formatter (name):
    return load_module("formatters", name)

def _load_highlighter (name):
    return load_module("highlighters", name)


def pretty_print (src, s, out, with_formatter, ext=None):
    """ `src' is a filepath to be formatted. `out' is a file object
        to be written."""

    if ext == "h":
        ext = "c" # XXX: Pygments does not highlight .h files
    elif ext.lower() == "s":
        ext = "asm" # XXX: Pygments does not highlight .S files
    elif ext is None:
        ext = guess_ext_by_filename(src)
        if ext == "":
            ext = guess_ext_by_contents(s)

    # format
    if with_formatter:
        f = _load_formatter(ext)
        if f is not None:
            ext,s = f.format(s)

    # highlight
    h = _load_highlighter(ext)
    if h is None:
        try:
            lexer = pygments.lexers.get_lexer_by_name(ext)
        except pygments.util.ClassNotFound:
            lexer = pygments.lexers.get_lexer_for_mimetype("text/plain")
        fmt = pygments.formatters.Terminal256Formatter(encoding="utf-8")
        pygments.highlight(s, lexer, fmt, out)
    else:
        h.highlight(out, s)
        out.close()
